@@ -1,8 +1,8 @@
 import os
 import sys
 
-from .fs import parent_of
 from .fstab import Fstab
+from .fstab import volume_of
 from .trash import EX_OK, EX_IOERR
 from .trash import TrashDirectories
 from .trash import backup_file_path_from
@@ -13,50 +13,61 @@ from datetime import datetime
 def main():
     return TrashPutCmd(
         sys.stdout,
-        sys.stderr
+        sys.stderr,
+        os.environ,
+        volume_of,
+        parent_path,
+        os.path.realpath
     ).run(sys.argv)
 
+def parent_path(path):
+    return os.path.realpath(os.path.dirname(path))
+
 class TrashPutCmd:
     def __init__(self,
                  stdout,
                  stderr,
-                 environ = os.environ,
-                 volume_of = Fstab().volume_of):
-        self.stdout    = stdout
-        self.stderr    = stderr
-        self.environ   = environ
-        self.volume_of = volume_of
-        self.logger    = MyLogger(self.stderr)
-        self.reporter  = TrashPutReporter(self.logger)
+                 environ,
+                 volume_of,
+                 parent_path,
+                 realpath):
+        self.stdout      = stdout
+        self.stderr      = stderr
+        self.environ     = environ
+        self.volume_of   = volume_of
+        self.fs          = RealFs()
+        self.getuid      = os.getuid
+        self.now         = datetime.now
+        self.parent_path = parent_path
+        self.realpath    = realpath
 
     def run(self, argv):
-        program_name = os.path.basename(argv[0])
-        self.logger.use_program_name(program_name)
+        program_name  = os.path.basename(argv[0])
+
+        logger = MyLogger(self.stderr, program_name)
 
         parser = self.get_option_parser(program_name)
         try:
             (options, args) = parser.parse_args(argv[1:])
-            if options.verbose: self.logger.be_verbose()
+            if options.verbose: logger.be_verbose()
 
             if len(args) <= 0:
                 parser.error("Please specify the files to trash.")
-        except SystemExit,e:
+        except SystemExit as e:
             return e.code
+        else:
+            reporter = TrashPutReporter(logger)
+            trashcan = GlobalTrashCan(reporter    = reporter,
+                                      volume_of   = self.volume_of,
+                                      environ     = self.environ,
+                                      fs          = self.fs,
+                                      getuid      = self.getuid,
+                                      now         = self.now,
+                                      parent_path = self.parent_path,
+                                      realpath    = self.realpath)
+            trashcan.trash_all(args)
 
-        self.trashcan = GlobalTrashCan(
-                reporter = self.reporter,
-                volume_of = self.volume_of,
-                environ = self.environ,
-                fs = RealFs(),
-                getuid = os.getuid,
-                now = datetime.now)
-        self.trash_all(args)
-
-        return self.reporter.exit_code()
-
-    def trash_all(self, args):
-        for arg in args :
-            self.trashcan.trash(arg)
+            return reporter.exit_code()
 
     def get_option_parser(self, program_name):
         from optparse import OptionParser
@@ -77,9 +88,9 @@ class TrashPutCmd:
                           help="ignored (for GNU rm compatibility)")
         parser.add_option("-v", "--verbose", action="store_true",
                           help="explain what is being done", dest="verbose")
+        original_print_help = parser.print_help
         def patched_print_help():
-            encoding = parser._get_encoding(self.stdout)
-            self.stdout.write(parser.format_help().encode(encoding, "replace"))
+            original_print_help(self.stdout)
         def patched_error(msg):
             parser.print_usage(self.stderr)
             parser.exit(2, "%s: error: %s\n" % (program_name, msg))
@@ -94,7 +105,7 @@ class TrashPutCmd:
         return parser
 
 epilog="""\
-To remove a file whose name starts with a `-', for example `-foo',
+To remove a file whose name starts with a '-', for example '-foo',
 use one of these commands:
 
     trash -- -foo
@@ -104,12 +115,10 @@ use one of these commands:
 Report bugs to https://github.com/andreafrancia/trash-cli/issues"""
 
 class MyLogger:
-    def __init__(self, stderr):
-        self.program_name = 'ERROR'
+    def __init__(self, stderr, program_name):
+        self.program_name = program_name
         self.stderr=stderr
         self.verbose = False
-    def use_program_name(self, program_name):
-        self.program_name = program_name
     def be_verbose(self):
         self.verbose = True
     def info(self,message):
@@ -131,17 +140,17 @@ class NullObject:
         return lambda *argl,**args:None
 
 class TrashPutReporter:
-    def __init__(self, logger = NullObject()):
+    def __init__(self, logger):
         self.logger = logger
         self.some_file_has_not_be_trashed = False
         self.no_argument_specified = False
     def unable_to_trash_dot_entries(self,file):
-        self.logger.warning("cannot trash %s `%s'" % (describe(file), file))
+        self.logger.warning("cannot trash %s '%s'" % (describe(file), file))
     def unable_to_trash_file(self,f):
-        self.logger.warning("cannot trash %s `%s'" % (describe(f), f))
+        self.logger.warning("cannot trash %s '%s'" % (describe(f), f))
         self.some_file_has_not_be_trashed = True
     def file_has_been_trashed_in_as(self, trashee, trash_directory, destination):
-        self.logger.info("`%s' trashed in %s" % (trashee,
+        self.logger.info("'%s' trashed in %s" % (trashee,
                                                  shrinkuser(trash_directory)))
     def found_unsercure_trash_dir_symlink(self, trash_dir_path):
         self.logger.info("found unsecure .Trash dir (should not be a symlink): %s"
@@ -157,11 +166,16 @@ class TrashPutReporter:
                                         trash_directory, error):
         self.logger.info("Failed to trash %s in %s, because :%s" % (
            file_to_be_trashed, shrinkuser(trash_directory), error))
+    def trash_dir_with_volume(self, trash_dir_path, volume_path):
+        self.logger.info("Trash-dir: %s from volume: %s" % (trash_dir_path,
+                                                            volume_path))
     def exit_code(self):
         if not self.some_file_has_not_be_trashed:
             return EX_OK
         else:
             return EX_IOERR
+    def volume_of_file(self,volume):
+        self.logger.info("Volume of file: %s" % volume)
 
 def describe(path):
     """
@@ -169,8 +183,8 @@ def describe(path):
     Options:
      - "symbolic link"
      - "directory"
-     - "`.' directory"
-     - "`..' directory"
+     - "'.' directory"
+     - "'..' directory"
      - "regular file"
      - "regular empty file"
      - "non existent"
@@ -185,9 +199,9 @@ def describe(path):
             return 'directory'
         else:
             if os.path.basename(path) == '.':
-                return "`.' directory"
+                return "'.' directory"
             elif os.path.basename(path) == '..':
-                return "`..' directory"
+                return "'..' directory"
             else:
                 return 'directory'
     elif os.path.isfile(path):
@@ -216,14 +230,20 @@ class GlobalTrashCan:
     class NullReporter:
         def __getattr__(self,name):
             return lambda *argl,**args:None
-    def __init__(self, environ, volume_of, reporter, fs, getuid, now):
-        self.getuid        = getuid
-        self.reporter      = reporter
-        self.volume_of     = volume_of
-        self.now           = now
-        self.fs            = fs
-        self.trash_directories = TrashDirectories(
-                self.volume_of, getuid, environ)
+    def __init__(self, environ, volume_of, reporter, fs, getuid, now,
+                 parent_path, realpath):
+        self.getuid            = getuid
+        self.reporter          = reporter
+        self.volume_of         = volume_of
+        self.now               = now
+        self.fs                = fs
+        self.environ           = environ
+        self.parent_path       = parent_path
+        self.realpath          = realpath
+
+    def trash_all(self, args):
+        for arg in args :
+            self.trash(arg)
 
     def trash(self, file) :
         """
@@ -245,12 +265,17 @@ class GlobalTrashCan:
             self.reporter.unable_to_trash_dot_entries(file)
             return
 
-        candidates = PossibleTrashDirectories(self.fs)
-        self._possible_trash_directories_for(file, candidates)
+        volume_of_file_to_be_trashed = self.volume_of_parent(file)
+        self.reporter.volume_of_file(volume_of_file_to_be_trashed)
+        candidates = self._possible_trash_directories_for(volume_of_file_to_be_trashed)
         file_has_been_trashed = False
         for trash_dir in candidates.trash_dirs:
             if self._is_trash_dir_secure(trash_dir):
-                if self._file_could_be_trashed_in(file, trash_dir.path):
+                volume_of_trash_dir = self.volume_of(self.realpath(trash_dir.path))
+                self.reporter.trash_dir_with_volume(trash_dir.path,
+                                                    volume_of_trash_dir)
+                if self._file_could_be_trashed_in(volume_of_file_to_be_trashed,
+                                                  volume_of_trash_dir):
                     try:
                         trashed_file = trash_dir.trash(file)
                         self.reporter.file_has_been_trashed_in_as(
@@ -259,7 +284,7 @@ class GlobalTrashCan:
                             trashed_file['where_file_was_stored'])
                         file_has_been_trashed = True
 
-                    except (IOError, OSError), error:
+                    except (IOError, OSError) as error:
                         self.reporter.unable_to_trash_file_in_because(
                                 file, trash_dir.path, str(error))
 
@@ -268,6 +293,9 @@ class GlobalTrashCan:
         if not file_has_been_trashed:
             self.reporter.unable_to_trash_file(file)
 
+    def volume_of_parent(self, file):
+        return self.volume_of(self.parent_path(file))
+
     def _is_trash_dir_secure(self, trash_dir):
         class ValidationOutput:
             def __init__(self):
@@ -275,15 +303,15 @@ class GlobalTrashCan:
             def not_valid_should_be_a_dir(_):
                 self.reporter.invalid_top_trash_is_not_a_dir(
                         os.path.dirname(trash_dir.path))
-                self.valid = False
+                _.valid = False
             def not_valid_parent_should_not_be_a_symlink(_):
                 self.reporter.found_unsercure_trash_dir_symlink(
                         os.path.dirname(trash_dir.path))
-                self.valid = False
+                _.valid = False
             def not_valid_parent_should_be_sticky(_):
                 self.reporter.found_unsecure_trash_dir_unsticky(
                         os.path.dirname(trash_dir.path))
-                self.valid = False
+                _.valid = False
             def is_valid(self):
                 self.valid = True
         output = ValidationOutput()
@@ -295,23 +323,23 @@ class GlobalTrashCan:
         basename = os.path.basename(file)
         return (basename == ".") or (basename == "..")
 
-    def _file_could_be_trashed_in(self,file_to_be_trashed,trash_dir_path):
-        return self.volume_of(trash_dir_path) == self.volume_of_parent(file_to_be_trashed)
+    def _file_could_be_trashed_in(self,
+                                  volume_of_file_to_be_trashed,
+                                  volume_of_trash_dir):
+        return volume_of_trash_dir == volume_of_file_to_be_trashed
 
-    def _possible_trash_directories_for(self, file, candidates):
-        volume = self.volume_of_parent(file)
-
-        self.trash_directories.home_trash_dir(
-                candidates.add_home_trash)
-        self.trash_directories.volume_trash_dir1(
-                volume, candidates.add_top_trash_dir)
-        self.trash_directories.volume_trash_dir2(
-                volume, candidates.add_alt_top_trash_dir)
+    def _possible_trash_directories_for(self, volume):
+        candidates = PossibleTrashDirectories(self.fs)
+        trash_directories = TrashDirectories(self.volume_of,
+                                             self.getuid,
+                                             self.environ)
+        trash_directories.home_trash_dir(candidates.add_home_trash)
+        trash_directories.volume_trash_dir1(volume,
+                                            candidates.add_top_trash_dir)
+        trash_directories.volume_trash_dir2(volume,
+                                            candidates.add_alt_top_trash_dir)
         return candidates
 
-    def volume_of_parent(self, file):
-        return self.volume_of(parent_of(file))
-
 class PossibleTrashDirectories:
     def __init__(self, fs):
         self.trash_dirs = []
@@ -332,6 +360,10 @@ class PossibleTrashDirectories:
     def _make_trash_dir(self, path, volume):
         return TrashDirectoryForPut(path, volume, datetime.now, self.fs)
 
+def parent_realpath(path):
+    parent = os.path.dirname(path)
+    return os.path.realpath(parent)
+
 class TrashDirectoryForPut:
     from datetime import datetime
     def __init__(self, path, volume, now, fs):
@@ -350,7 +382,7 @@ class TrashDirectoryForPut:
         self.remove_file  = fs.remove_file
         self.ensure_dir   = fs.ensure_dir
 
-        self.path_for_trash_info = OriginalLocation(os.path.realpath)
+        self.path_for_trash_info = OriginalLocation(parent_realpath)
 
     def store_absolute_paths(self):
         self.path_for_trash_info.make_absolutes_paths()
@@ -386,15 +418,18 @@ class TrashDirectoryForPut:
         def format_date(deletion_date):
             return deletion_date.strftime("%Y-%m-%dT%H:%M:%S")
         def format_original_location(original_location):
-            import urllib
-            return urllib.quote(original_location,'/')
+            try:
+                from urllib import quote
+            except ImportError:
+                from urllib.parse import quote
+            return quote(original_location,'/')
         content = ("[Trash Info]\n" +
                    "Path=%s\n" % format_original_location(original_location) +
-                   "DeletionDate=%s\n" % format_date(deletion_date))
+                   "DeletionDate=%s\n" % format_date(deletion_date)).encode('utf-8')
         return content
 
     def ensure_files_dir_exists(self):
-        self.ensure_dir(self.files_dir, 0700)
+        self.ensure_dir(self.files_dir, 0o700)
 
     def persist_trash_info(self, info_dir, basename, content) :
         """
@@ -402,7 +437,7 @@ class TrashDirectoryForPut:
         returns the created TrashInfoFile.
         """
 
-        self.ensure_dir(info_dir, 0700)
+        self.ensure_dir(info_dir, 0o700)
 
         # write trash info
         index = 0
@@ -462,8 +497,8 @@ class TopTrashDirWriteRules:
         output.is_valid()
 
 class OriginalLocation:
-    def __init__(self, realpath):
-        self.realpath = realpath
+    def __init__(self, parent_realpath):
+        self.parent_realpath = parent_realpath
         self.make_absolutes_paths()
 
     def make_paths_relatives_to(self, topdir):
@@ -476,16 +511,12 @@ class OriginalLocation:
         self.normalized_path = os.path.normpath(path)
 
         basename = os.path.basename(self.normalized_path)
-        parent   = self._real_parent()
+        parent   = self.parent_realpath(self.normalized_path)
 
         parent = self.path_maker.calc_parent_path(parent)
 
         return os.path.join(parent, basename)
 
-    def _real_parent(self):
-        parent = os.path.dirname(self.normalized_path)
-        return self.realpath(parent)
-
 class TopDirRelativePaths:
     def __init__(self, topdir):
         self.topdir = topdir
