@@ -1,7 +1,7 @@
 # Copyright (C) 2007-2011 Andrea Francia Trivolzio(PV) Italy
 from __future__ import absolute_import
 
-version='0.16.12.26'
+version='0.17.1.14'
 
 import os
 import logging
@@ -17,7 +17,6 @@ EX_IOERR = getattr(os, 'EX_IOERR', 74)
 
 from .fs import list_files_in_dir
 import os
-from .fs import remove_file
 
 class TrashDirectory:
     def __init__(self, path, volume):
@@ -75,82 +74,7 @@ class TrashDirectories:
             path   = os.path.join(volume, ".Trash-%s" % self.getuid()),
             volume = volume)
 
-from .fs import FileSystemReader, contents_of, FileRemover
-
-class ListCmd:
-    def __init__(self, out, err, environ, list_volumes, getuid,
-                 file_reader   = FileSystemReader(),
-                 version       = version):
-
-        self.output      = self.Output(out, err)
-        self.err         = self.output.err
-        self.contents_of = file_reader.contents_of
-        self.version     = version
-        top_trashdir_rules = TopTrashDirRules(file_reader)
-        self.trashdirs = TrashDirs(environ, getuid,
-                                   list_volumes = list_volumes,
-                                   top_trashdir_rules=top_trashdir_rules)
-        self.harvester = Harvester(file_reader)
-
-    def run(self, *argv):
-        parse=Parser()
-        parse.on_help(PrintHelp(self.description, self.output.println))
-        parse.on_version(PrintVersion(self.output.println, self.version))
-        parse.as_default(self.list_trash)
-        parse(argv)
-    def list_trash(self):
-        self.harvester.on_volume = self.output.set_volume_path
-        self.harvester.on_trashinfo_found = self._print_trashinfo
-
-        self.trashdirs.on_trashdir_skipped_because_parent_not_sticky = self.output.top_trashdir_skipped_because_parent_not_sticky
-        self.trashdirs.on_trashdir_skipped_because_parent_is_symlink = self.output.top_trashdir_skipped_because_parent_is_symlink
-        self.trashdirs.on_trash_dir_found = self.harvester._analize_trash_directory
-
-        self.trashdirs.list_trashdirs()
-    def _print_trashinfo(self, path):
-        try:
-            contents = self.contents_of(path)
-        except IOError as e :
-            self.output.print_read_error(e)
-        else:
-            deletion_date = parse_deletion_date(contents) or unknown_date()
-            try:
-                path = parse_path(contents)
-            except ParseError:
-                self.output.print_parse_path_error(path)
-            else:
-                self.output.print_entry(deletion_date, path)
-    def description(self, program_name, printer):
-        printer.usage('Usage: %s [OPTIONS...]' % program_name)
-        printer.summary('List trashed files')
-        printer.options(
-           "  --version   show program's version number and exit",
-           "  -h, --help  show this help message and exit")
-        printer.bug_reporting()
-    class Output:
-        def __init__(self, out, err):
-            self.out = out
-            self.err = err
-        def println(self, line):
-            self.out.write(line+'\n')
-        def error(self, line):
-            self.err.write(line+'\n')
-        def print_read_error(self, error):
-            self.error(str(error))
-        def print_parse_path_error(self, offending_file):
-            self.error("Parse Error: %s: Unable to parse Path." % (offending_file))
-        def top_trashdir_skipped_because_parent_not_sticky(self, trashdir):
-            self.error("TrashDir skipped because parent not sticky: %s"
-                    % trashdir)
-        def top_trashdir_skipped_because_parent_is_symlink(self, trashdir):
-            self.error("TrashDir skipped because parent is symlink: %s"
-                    % trashdir)
-        def set_volume_path(self, volume_path):
-            self.volume_path = volume_path
-        def print_entry(self, maybe_deletion_date, relative_location):
-            import os
-            original_location = os.path.join(self.volume_path, relative_location)
-            self.println("%s %s" %(maybe_deletion_date, original_location))
+from .fs import FileSystemReader, contents_of
 
 def do_nothing(*argv, **argvk): pass
 class Parser:
@@ -170,14 +94,17 @@ class Parser:
             options, arguments = getopt(argv[1:],
                                         self.short_options,
                                         self.long_options)
-        except GetoptError, e:
+        except GetoptError as e:
             invalid_option = e.opt
             self._on_invalid_option(program_name, invalid_option)
         else:
             for option, value in options:
-                if option in self.actions:
+                if option in ('--help', '-h', '--version'):
                     self.actions[option](program_name)
                     return
+                if option in self.actions:
+                    self.actions[option](value)
+                    return
             for argument in arguments:
                 self.argument_action(argument)
             self.default_action()
@@ -193,6 +120,9 @@ class Parser:
 
     def add_option(self, long_option, action, short_aliases=''):
         self.long_options.append(long_option)
+        if long_option.endswith('='):
+            import re
+            long_option = re.sub('=$', '', long_option)
         self.actions['--' + long_option] = action
         for short_alias in short_aliases:
             self.add_short_option(short_alias, action)
@@ -206,42 +136,6 @@ class Parser:
     def as_default(self, default_action):
         self.default_action = default_action
 
-class CleanableTrashcan:
-    def __init__(self, file_remover):
-        self._file_remover = file_remover
-    def delete_orphan(self, path_to_backup_copy):
-        self._file_remover.remove_file(path_to_backup_copy)
-    def delete_trashinfo_and_backup_copy(self, trashinfo_path):
-        backup_copy = self._path_of_backup_copy(trashinfo_path)
-        self._file_remover.remove_file_if_exists(backup_copy)
-        self._file_remover.remove_file(trashinfo_path)
-    def _path_of_backup_copy(self, path_to_trashinfo):
-        from os.path import dirname as parent_of, join, basename
-        trash_dir = parent_of(parent_of(path_to_trashinfo))
-        return join(trash_dir, 'files', basename(path_to_trashinfo)[:-len('.trashinfo')])
-
-class ExpiryDate:
-    def __init__(self, contents_of, now, trashcan):
-        self._contents_of  = contents_of
-        self._now          = now
-        self._maybe_delete = self._delete_unconditionally
-        self._trashcan = trashcan
-    def set_max_age_in_days(self, arg):
-        self.max_age_in_days = int(arg)
-        self._maybe_delete = self._delete_according_date
-    def delete_if_expired(self, trashinfo_path):
-        self._maybe_delete(trashinfo_path)
-    def _delete_according_date(self, trashinfo_path):
-        contents = self._contents_of(trashinfo_path)
-        ParseTrashInfo(
-            on_deletion_date=IfDate(
-                OlderThan(self.max_age_in_days, self._now),
-                lambda: self._delete_unconditionally(trashinfo_path)
-            ),
-        )(contents)
-    def _delete_unconditionally(self, trashinfo_path):
-        self._trashcan.delete_trashinfo_and_backup_copy(trashinfo_path)
-
 class TrashDirs:
     def __init__(self, environ, getuid, list_volumes, top_trashdir_rules):
         self.getuid             = getuid
@@ -280,88 +174,20 @@ class TrashDirs:
         self.on_trash_dir_found(alt_top_trashdir, volume)
 
 from datetime import datetime
-class EmptyCmd:
-    def __init__(self, out, err, environ, list_volumes,
-                 now           = datetime.now,
-                 file_reader   = FileSystemReader(),
-                 getuid        = os.getuid,
-                 file_remover  = FileRemover(),
-                 version       = version):
-
-        self.out          = out
-        self.err          = err
-        self.file_reader  = file_reader
-        top_trashdir_rules = TopTrashDirRules(file_reader)
-        self.trashdirs = TrashDirs(environ, getuid,
-                                   list_volumes = list_volumes,
-                                   top_trashdir_rules = top_trashdir_rules)
-        self.harvester = Harvester(file_reader)
-        self.version      = version
-        self._cleaning    = CleanableTrashcan(file_remover)
-        self._expiry_date = ExpiryDate(file_reader.contents_of, now,
-                                       self._cleaning)
-
-    def run(self, *argv):
-        self.exit_code     = EX_OK
-
-        parse = Parser()
-        parse.on_help(PrintHelp(self.description, self.println))
-        parse.on_version(PrintVersion(self.println, self.version))
-        parse.on_argument(self._expiry_date.set_max_age_in_days)
-        parse.as_default(self._empty_all_trashdirs)
-        parse.on_invalid_option(self.report_invalid_option_usage)
-
-        parse(argv)
-
-        return self.exit_code
-
-    def report_invalid_option_usage(self, program_name, option):
-        self.err.write(
-            "{program_name}: invalid option -- '{option}'\n".format(**locals()))
-        self.exit_code |= EX_USAGE
-
-    def description(self, program_name, printer):
-        printer.usage('Usage: %s [days]' % program_name)
-        printer.summary('Purge trashed files.')
-        printer.options(
-           "  --version   show program's version number and exit",
-           "  -h, --help  show this help message and exit")
-        printer.bug_reporting()
-    def _empty_all_trashdirs(self):
-        self.harvester.on_trashinfo_found = self._expiry_date.delete_if_expired
-        self.harvester.on_orphan_found = self._cleaning.delete_orphan
-        self.trashdirs.on_trash_dir_found = self.harvester._analize_trash_directory
-        self.trashdirs.list_trashdirs()
-    def println(self, line):
-        self.out.write(line + '\n')
 
 class Harvester:
     def __init__(self, file_reader):
         self.file_reader = file_reader
-        self.trashdir = TrashDir(self.file_reader)
 
         self.on_orphan_found                               = do_nothing
         self.on_trashinfo_found                            = do_nothing
         self.on_volume                                     = do_nothing
-    def _analize_trash_directory(self, trash_dir_path, volume_path):
+    def analize_trash_directory(self, trash_dir_path, volume_path):
         self.on_volume(volume_path)
-        self.trashdir.open(trash_dir_path, volume_path)
-        self.trashdir.each_trashinfo(self.on_trashinfo_found)
-        self.trashdir.each_orphan(self.on_orphan_found)
-
-class IfDate:
-    def __init__(self, date_criteria, then):
-        self.date_criteria = date_criteria
-        self.then          = then
-    def __call__(self, date2):
-        if self.date_criteria(date2):
-            self.then()
-class OlderThan:
-    def __init__(self, days_ago, now):
-        from datetime import timedelta
-        self.limit_date = now() - timedelta(days=days_ago)
-    def __call__(self, deletion_date):
-        return deletion_date < self.limit_date
+        trashdir = TrashDir(self.file_reader)
+        trashdir.open(trash_dir_path, volume_path)
+        trashdir.each_trashinfo(self.on_trashinfo_found)
+        trashdir.each_orphan(self.on_orphan_found)
 
 class PrintHelp:
     def __init__(self, description, println):
@@ -465,6 +291,11 @@ def maybe_parse_deletion_date(contents):
 def unknown_date():
     return '????-??-?? ??:??:??'
 
+try:
+    from urllib import unquote
+except ImportError:
+    from urllib.parse import unquote
+
 class ParseTrashInfo:
     def __init__(self,
                  on_deletion_date = do_nothing,
@@ -475,7 +306,6 @@ class ParseTrashInfo:
         self.found_path = on_path
     def __call__(self, contents):
         from datetime import datetime
-        import urllib
         for line in contents.split('\n'):
             if line.startswith('DeletionDate='):
                 try:
@@ -486,7 +316,7 @@ class ParseTrashInfo:
                     self.found_deletion_date(date)
 
             if line.startswith('Path='):
-                path=urllib.unquote(line[len('Path='):])
+                path=unquote(line[len('Path='):])
                 self.found_path(path)
 
 class Basket:
@@ -500,9 +330,22 @@ def parse_deletion_date(contents):
     return result.collected
 
 def parse_path(contents):
-    import urllib
     for line in contents.split('\n'):
         if line.startswith('Path='):
-            return urllib.unquote(line[len('Path='):])
+            return unquote(line[len('Path='):])
     raise ParseError('Unable to parse Path')
 
+class CleanableTrashcan:
+    def __init__(self, file_remover):
+        self._file_remover = file_remover
+    def delete_orphan(self, path_to_backup_copy):
+        self._file_remover.remove_file(path_to_backup_copy)
+    def delete_trashinfo_and_backup_copy(self, trashinfo_path):
+        backup_copy = self._path_of_backup_copy(trashinfo_path)
+        self._file_remover.remove_file_if_exists(backup_copy)
+        self._file_remover.remove_file(trashinfo_path)
+    def _path_of_backup_copy(self, path_to_trashinfo):
+        from os.path import dirname, join, basename
+        trash_dir = dirname(dirname(path_to_trashinfo))
+        return join(trash_dir, 'files', basename(path_to_trashinfo)[:-len('.trashinfo')])
+
